(* type width *)
type width = int
(* type height *)
type height = int
(* type t *)
type t = (width * height * Strng.t list)
(* height : (width * height * Strng.t list) -> height *)
let e = (0,0,[[]])
let height (_,h,_) = h
(* width : (width * height * Strng.t list) -> width *)
let width (w,_,_) = w
(* above : t -> t -> t *)
let above (w1,h1,p1) (w2,h2,p2) =
 if w1 = w2 then (w1,h1+h2,p1@p2)
            else failwith "different widths"
(* beside : t -> t -> t *)
let beside (w1,h1,p1) (w2,h2,p2) =
 if h1 = h2 then (w1+w2,h1,Lst.zip_with (@) p1 p2)
            else failwith "different heights"
(* to_strng : t -> Strng.t *)
let to_strng (_,_,p) = Lst.join ['\n'] p 
(* stack : t list -> t *)
let stack ps = Lst.foldr1 above ps
(* spread : t list -> t *)
let spread ps = Lst.foldr1 beside ps
(* pixel : char -> t *)
let pixel c = (1,1,[[c]])
(* row : char list -> t *)
let row s = spread(Lst.map pixel s)
(* column : char list -> t *)
let column s = stack(Lst.map pixel s)
(* empty : width -> height -> t *)
let empty w h =
 let line = Lst.replicate w ' ' in
 let rows = Lst.replicate h line in
 stack(Lst.map row rows)
(* stack_with : height -> t list -> t *)
let stack_with h ps = Lst.foldr1 (fun p q ->
 above (above p (empty (width q) h)) q) ps
(* spread_with : width -> t list -> t *)
let spread_with w ps = Lst.foldr1 (fun p q ->
 beside (beside p (empty w (height q))) q) ps
(* tile : t list list -> t *)
let tile pss = stack(Lst.map spread pss)
(* tile_with : width -> height -> t list list -> t *)
let tile_with w h pss = 
 stack_with h (Lst.map (spread_with w) pss)
(* toplevel_printer : Format.formatter -> t -> unit *)
let toplevel_printer fmt ((_,_,lines):t) =
 Format.fprintf fmt "\n";
 List.iter (fun line ->
  List.iter (Format.fprintf fmt "%c") line; Format.fprintf fmt "\n"
 ) lines
(* E03 *)
(* paragraph : int -> string -> t *)
let paragraph i s = 
 let rec p i s =
  if Lst.length s <= i then row (Strng.left_justify i s)
  else above (row (Lst.take i s)) (p i (Lst.drop i s))
 in p i (Strng.of_string s)
(* paragraph2 : int -> (int -> Strng.t -> Strng.t) -> string -> t *)
let paragraph2 i justify s =
 let ls = Strng.lines i (Strng.of_string s) in
 stack (Lst.map (fun l -> row (justify i l)) ls)
(* mirror_v : t -> t *)
let rec mirror_v (w,h,pss) = (w,h,Lst.map Lst.rev pss)
(* mirror_h : t -> t *)
let rec mirror_h (w,h,pss) = (w,h,Lst.rev pss)
(* mirror : t -> t *)
let mirror p = mirror_h (mirror_v p)
(* empty_fixed : int -> int -> t*)
let empty w h = 
 if w = 0 then (0,h,Lst.replicate h []) 
 else if h = 0 then (w,0,[]) 
 else empty w h
(* rotate : t -> t *)
let rotate (w,_,pss) = Lst.foldr (fun l p -> beside p (column l)) (empty 0 w) pss
(* make : int -> int -> char -> t *)
let make w h c = 
 let line = Lst.replicate w c in
 let rows = Lst.replicate h line in
 stack(Lst.map row rows)
(* frame : char -> t -> t *)
let frame c p = 
 let h = make (width p + 2) 1 c in
 let v = make 1 (height p) c in
 stack [h;spread [v;p;v];h]
(*  *)

