let months = ["January"; "February"; "March"; "April"; "May"; "June";
  "July"; "August"; "September"; "October"; "November"; "December"];;

let week = Strng.of_string "Su Mo Tu We Th Fr Sa";;

(* should be in Lst.ml *)
let rec range n m = if n < m then n :: range (n+1) m else [];;

let cal m y =
  (* total width is 20 *)
  let w = 20 in
  (* get month name *)
  let month = Strng.of_string (Lst.nth months (m-1)) in
  (* format year as string *)
  let year = Ex3_7.to_strng y 10 in
  (* first line, e.g. January 2007, centered *)
  let head = Strng.center w (month @ [' '] @ year) in
  (* first day of week (0 = Sunday) *)
  let start = Lst.nth (Calendar.first_days y) (m-1) in
  (* length of month *)
  let length = Lst.nth (Calendar.month_lengths y) (m-1) in
  (* list of days prefixed with empty entries for the days of the
     first week that are in the previous month, e.g.
     ["   ";"  1";"  2";...;" 31"] *)
  let days = Lst.replicate start (Strng.of_string "   ") @
        Lst.map (fun d -> Strng.right_justify 3 (Ex3_7.to_strng d 10))
                 (range 1 (length+1)) in
  (* number of rows *)
  let rows = (Lst.length days + 6) / 7 in
  (* collect head, the week day header, and the rows for the days
     of the month; note that concatenating 7 day numbers produces
     an l-string of length 21; the Lst.drop 1 gets rid of the first
     character (which is always a space) *)
  let lines = head :: week ::
     Lst.map (fun i -> Strng.left_justify w (
          Lst.drop 1 (Lst.concat (Lst.take 7 (Lst.drop (i*7) days)))))
       (range 0 rows) in
  (* finally, stack the rows *)
  Picture.stack (List.map Picture.row lines)
;;

     
